#include "addressbookdatabase.h"

#include <string>
#include <qstring.h>
#include <qdir.h>
#include <kdebug.h>

#include <addressbooksyncee.h>
#include <kabc/vcardconverter.h> 
#include <konnector.h>

#include <synchistory.h>

#include "syncmldevinf.h"
#include "syncml.h"
#include "syncmldatabase.h"
#include "mlsessionhandler.h"

#include "mlsynchistory.h"

using namespace KSyncML;

class AddressBookHistory : public KSyncML::SyncHistory<KSync::AddressBookSyncee, KSync::AddressBookSyncEntry> 
//class AddressBookHistory : public KSync::SyncHistory<KSync::AddressBookSyncee, KSync::AddressBookSyncEntry> 
{
	public:
		AddressBookHistory( const QString& str );
		virtual ~AddressBookHistory();
		virtual QString string( KSync::AddressBookSyncEntry * );
};


AddressBookHistory::AddressBookHistory( const QString& file )
	: KSyncML::SyncHistory<KSync::AddressBookSyncee, KSync::AddressBookSyncEntry>( file )
{ 
}

AddressBookHistory::~AddressBookHistory()
{
}



QString AddressBookHistory::string( KSync::AddressBookSyncEntry* entry) 
{
	KABC::Addressee adr = entry->addressee();

	//kdDebug() << "AddressBookHistory: Correctly converting '" << adr.assembledName() << "' to string" << endl;
	
	QString str;
	str = adr.givenName();
	str += adr.additionalName();
	str += adr.familyName();
	str += adr.suffix();
	str += adr.role();
	str += adr.organization();


    /*
	* busines numbers
	*/
	KABC::PhoneNumber number = adr.phoneNumber( KABC::PhoneNumber::Work );
	str += number.number();

	number = adr.phoneNumber( KABC::PhoneNumber::Work | KABC::PhoneNumber::Fax );
	str += number.number();

	number = adr.phoneNumber( KABC::PhoneNumber::Work | KABC::PhoneNumber::Cell );
	str += number.number();

	str += adr.preferredEmail();
	str += adr.emails().join(";");

    /*
	* home/private numbers
	*/
	number = adr.phoneNumber( KABC::PhoneNumber::Home );
	str += number.number();

	number = adr.phoneNumber( KABC::PhoneNumber::Home | KABC::PhoneNumber::Fax );
	str += number.number();

	number = adr.phoneNumber( KABC::PhoneNumber::Cell );
	str += number.number();

    /*
	* address bits
	*/
	KABC::Address ad = adr.address( KABC::Address::Work );
	str += ad.street();
	str += ad.locality();
	str += ad.postalCode();
	str += ad.country();
	str += ad.region();


    /*
	* home address
	*/
	ad = adr.address( KABC::Address::Home );
	str += ad.street();
	str += ad.locality();
	str += ad.region();
	str += ad.postalCode();
	str += ad.country();

	str += adr.custom( "KADDRESSBOOK", "X-Department" );
	str += adr.custom( "KADDRESSBOOK", "X-SpouseName" );
	str += adr.custom( "KADDRESSBOOK", "X-Office" );
	str += adr.custom( "KADDRESSBOOK", "X-Profession" );
	str += adr.custom( "KADDRESSBOOK", "AssistantsName" );
	str += adr.custom( "KADDRESSBOOK", "ManagersName" );
	str += adr.custom( "opie", "Children" );
	str += adr.custom( "opie", "HomeWebPage" );
	str += adr.custom( "opie", "Gender" );
//    str += adr.custom( "opie", "Birthday" );
	str += adr.birthday().date().toString(Qt::ISODate);
	str += adr.custom( "KADDRESSBOOK", "X-Anniversary" );

	str += adr.note();
	str += adr.nickName();
	str += adr.categories().join(";");


	return KSync::Konnector::generateMD5Sum( str );
}



AddressBookDatabase::AddressBookDatabase() : SyncMLDatabase("./addressbook")
{
	dataStore.SourceRef="./addressbook";
	dataStore.DisplayName="Contact Database";
	dataStore.Rx.insert(pair<string,string>("text/x-vcard","2.1"));
	dataStore.RxPref = pair<string,string>("text/x-vcard","2.1");
	dataStore.Tx.insert(pair<string,string>("text/x-vcard","2.1"));
	dataStore.TxPref = pair<string,string>("text/x-vcard","2.1");
	dataStore.SyncCap[SMLSyncType::twoway] = false;///XXX TODO hmm... will it work?
	dataStore.SyncCap[SMLSyncType::slow] = true;
}


AddressBookDatabase::~AddressBookDatabase()
{
}
void AddressBookDatabase::processClientData(SyncMLSession::TSynceesList& data)
{
	//
	// Ahhhh.... damn!
	// I have finally understood why this didn't work. In fact, this client/server
	// key thing is really complicated in SyncML. It's because the LocURI/RemoteURI
	// has nothing to do with the actual UID of data (which can be different too)
	// Therefore we need a local dictionary LocURI -> UID

	kdDebug() << "*********** AddressBookDatabase::processClientData data count = " << data.count() << endl;
	KABC::VCardConverter converter;
	
	if (syncee)
		syncee->reset();
	else
	{
		syncee = new KSync::AddressBookSyncee();
		syncee->setTitle("SyncML");
	}
	
	SyncMLSession::TSynceesList::iterator it;

	for ( it = data.begin(); it != data.end(); ++it )
	{
		SyncMLItem &item = *it;
		
		KABC::Addressee adressee = converter.parseVCard(item.item);
		
		if (adressee.assembledName() != "")
		{
			//this is needed for P900
			QString name = adressee.assembledName();
			adressee.setFormattedName(name);
			
			KSync::AddressBookSyncEntry * entry = new KSync::AddressBookSyncEntry  ( adressee, syncee );
			
			remoteURI_UID_Dict[entry->id()] = item.ID;
			
			syncee->addEntry(entry);
					
			item.syncEntryID = entry->id();
			items.append(&item);
			
			kdDebug() << "AddressBookDatabase: got vcard " 
				<< adressee.uid() <<  " of '"
				<< adressee.assembledName() << "' (itemID = '" << item.ID << "')" << endl;
				//<< "', msgRef='" << item.msgRef << "', cmdRef='" << item.cmdRef << "')" << endl;
		}
		else
		{
			kdDebug() << "AddressBookDatabase: BUG in the vcard parser! got vcard " 
					<< adressee.uid() <<  " of a person with no name?" << endl;
			
			adressee.setFormattedName("Fake User");
			adressee.setUid(item.ID);
			
			KSync::AddressBookSyncEntry * entry = new KSync::AddressBookSyncEntry  ( adressee, syncee );
			
			remoteURI_UID_Dict[entry->id()] = item.ID;
			
			syncee->addEntry(entry);
					
			item.syncEntryID = entry->id();
			items.append(&item);
		}
	}
	
	//
	// Updating metadata and keys.
	QString histPath = m_databaseDir + "/addrdata";
	AddressBookHistory syncInfo( histPath );
	syncInfo.setSyncee((KSync::AddressBookSyncee *)syncee);
	syncInfo.processRemoteData(remoteURI_UID_Dict);
	
	_gotData = true;
}

ChangeLog *  AddressBookDatabase::getDBChangeLog()
{
	if (!_gotData)
		return NULL;
	
	
	ChangeLog * changeLog = new ChangeLog();
	changeLog->clear();
	//m_tempKeyDatabase.clear();
	
	//int counter = 0;
	//for (entry = (KSync::AddressBookSyncEntry *)syncee->firstEntry(); entry; entry = (KSync::AddressBookSyncEntry *)syncee->nextEntry())
	//  counter++;
	//kdDebug() << "*********** AddressBookDatabase::writeData data count = " << counter << endl;
	
	
	SyncMLItem *item = NULL;
	QValueList<SyncMLItem *>::iterator it;
	KSync::AddressBookSyncEntry * entry;
	
	//int tempGUID = 2000;//arbitrary value
	
	//
	// Updating metadata and keys.
	QString histPath = m_databaseDir + "/addrdata";
	AddressBookHistory syncInfo( histPath );
	syncInfo.setSyncee((KSync::AddressBookSyncee *)syncee);
	syncInfo.processLocalData(remoteURI_UID_Dict);
	
	//
	// syncee is now synchronized with the rest of Kitchensync. We need to find out the changes
	// made and upload everything to the client.
	for (entry = (KSync::AddressBookSyncEntry *)syncee->firstEntry(); entry; entry = (KSync::AddressBookSyncEntry *)syncee->nextEntry())
	{
		KABC::Addressee addressee = entry->addressee();
		kdDebug() << "AddressBookDatabase: write vcard "
			  << addressee.uid() <<  " of '" 
			   << addressee.assembledName() << "' (entryID='" << entry->id() << "', state='" << entry->state() << "')" << endl;
		
		
		item = NULL;
		bool notFound = true;
		//we look for the local entry in the item list
		for ( it = items.begin(); (notFound) && (it != items.end()); ++it )
		{
			item = (*it);
			QString savedID = item->syncEntryID;
			
			if (savedID == entry->id())
			{
				kdDebug() << "AddressBookDatabase: item->syncEntryID exists for this vcard" << endl;
				notFound = false;
			}
		}
		
		
		if (notFound)
		{
			//We only add the items that has a name (kitchensync creates some spurious vcards, i have no idea why)
			if (addressee.assembledName() != "")
			{
				kdDebug() << "AddressBookDatabase: Adding a new vcard for " << addressee.assembledName() << endl;
				KABC::VCardConverter converter;
				QString vcard = converter.createVCard( addressee, KABC::VCardConverter::v2_1);
			
				SyncMLItem * item = new SyncMLItem();
			
				//item->syncEntry = NULL;
			
				item->ID = addressee.uid();
				item->source = remoteURI_UID_Dict[entry->id()]; //QString::number(tempGUID);
			
				item->target = "";//"./addressbook";
			
				item->item = vcard;
				item->action = SyncMLItem::ADDED;
			
				ChangeLog::Record * r = new ChangeLog::Record(ChangeLog::ADD, item);
				changeLog->addRecord(r);
			
				//tempGUID++;
			}
			else
			{
				kdWarning(5201) << "AddressBookDatabase: Spurious vcard of id '" << addressee.uid() << "' detected" << endl;
			}
		}
		else
		{
		
			switch (entry->state())
			{
				case KSync::SyncEntry::Undefined:
				{
					if (item)
					{
						ChangeLog::Record * r = new ChangeLog::Record(ChangeLog::OK, item);
						changeLog->addRecord(r);
					}
					else
						kdDebug() << "AddressBookDatabase: BUG. Undefined but no item found" << endl;
					
					break;
				}
				case KSync::SyncEntry::Modified:
				{
					if (item)
					{
					  //XXX FIXME: we don't need "newItem", we can simply reuse "item" -> check it.
						SyncMLItem * newItem = new SyncMLItem();
					
						newItem->ID = item->ID;
						newItem->source = item->source;
						newItem->target = item->source;
						newItem->cmdRef = item->cmdRef;
						newItem->msgRef = item->msgRef;
						newItem->syncEntryID = item->syncEntryID;

						KABC::VCardConverter converter;
						QString vcard = converter.createVCard( addressee, KABC::VCardConverter::v2_1);

						newItem->item = vcard;
						newItem->action = SyncMLItem::REPLACED;
				
						ChangeLog::Record * r = new ChangeLog::Record(ChangeLog::REPLACE, newItem);
						changeLog->addRecord(r);
					}
					else
						kdDebug() << "AddressBookDatabase: BUG. Undefined but no item found" << endl;
					
					break;
				}
				case KSync::SyncEntry::Removed:
				{
					kdWarning() << "AddressBookDatabase: REMOVED NOT IMPLEMENTED YET" << endl;
					break;
				}
				case KSync::SyncEntry::Added:
				{
					ChangeLog::Record * r = new ChangeLog::Record(ChangeLog::OK, item);
					changeLog->addRecord(r);
					break;
				}
			}
		}
	}

	
	kdDebug() << "********************************" << endl;
	kdDebug() << "* ChangeLog database ADDRESSBOOK" << endl;
	changeLog->dump();
	kdDebug() << "********************************" << endl;
	
	return changeLog;
}


void AddressBookDatabase::processClientKeys(SyncMLSession::TKeysList& data)
{
	QString histPath = m_databaseDir + "/addrdata";
	AddressBookHistory syncInfo( histPath );
	syncInfo.processRemoteKeys(data);
}


