#include "calendardatabase.h"

#include <qstring.h>
#include <qdir.h>
#include <kdebug.h>

#include <calendarsyncee.h>
#include <libkcal/calendarlocal.h>
#include <libkcal/vcalformat.h>

#include <libkdepim/kpimprefs.h>

//#include <konnector.h>

#include <synchistory.h>

#include "syncmldevinf.h"
#include "syncml.h"
#include "syncmldatabase.h"
#include "mlsessionhandler.h"

#include "mlsynchistory.h"

using namespace KSyncML;
using namespace KCal;


CalendarDatabase::CalendarDatabase():SyncMLDatabase("./calendar")
{
	dataStore.SourceRef="./calendar";
	dataStore.DisplayName="Calendar";
	dataStore.Rx.insert(pair<string,string>("text/calendar","2.0"));
	dataStore.RxPref = pair<string,string>("text/calendar","2.0");
	dataStore.Tx.insert(pair<string,string>("text/calendar","2.0"));
	dataStore.TxPref = pair<string,string>("text/calendar","2.0");
	dataStore.SyncCap[SMLSyncType::twoway] = false;///XXX TODO hmm... will it work?
	dataStore.SyncCap[SMLSyncType::slow] = true;
}

CalendarDatabase::~CalendarDatabase()
{
}


void CalendarDatabase::processClientData(SyncMLSession::TSynceesList& data)
{
	kdDebug() << "*********** AddressBookDatabase::processClientData data count = " << data.count() << endl;
	
	
	CalendarLocal *tmpCalendar = new CalendarLocal(KPimPrefs::timezone());	
	delete syncee;
	syncee = new KSync::CalendarSyncee(tmpCalendar);
	syncee->setTitle("SyncML");
	syncee->setIdentifier("SyncMLCalendar");

	SyncMLSession::TSynceesList::iterator it;

	for ( it = data.begin(); it != data.end(); ++it )
	{
		SyncMLItem &item = *it;

		//kdDebug() << "XXXXXXXXXXXXXXXXX " << endl;
		//kdDebug() << item.item << endl;
		//kdDebug() << "XXXXXXXXXXXXXXXXX " << endl;

		KCal::CalendarLocal *tempCalendar = new KCal::CalendarLocal(KPimPrefs::timezone());
		
		VCalFormat vCal;
		CalendarLocal cal( KPimPrefs::timezone() );
		vCal.fromString(tempCalendar, item.item);
		
		/*Get All the events from the received calendar: They are only 1 !!! */
		Incidence::List incidencesList = tempCalendar->incidences();
		Incidence::List::Iterator it;
		kdDebug() << "There are " << incidencesList.size() << " events" << endl;
		for ( it = incidencesList.begin(); it != incidencesList.end(); ++it ) {

			KSync::CalendarSyncEntry* entry = new KSync::CalendarSyncEntry(*it , syncee);
			syncee->addEntry(entry);
			
			remoteURI_UID_Dict[entry->id()] = item.ID;
			item.syncEntryID = entry->id();
			items.append(&item);
		
			kdDebug() << "CalendarDatabase: got incidence " 
					<< (*it)->summary()
					<< "(itemID = '" << item.ID << "')" << endl;
		}
	}
	_gotData = true;
}

void CalendarDatabase::processClientKeys(SyncMLSession::TKeysList& data)
{
}

ChangeLog * CalendarDatabase::getDBChangeLog()
{
	if (!_gotData)
		return NULL;
	
	KSync::CalendarSyncEntry* entry;
	for (entry = (KSync::CalendarSyncEntry *)syncee->firstEntry(); entry; entry = (KSync::CalendarSyncEntry *)syncee->nextEntry())
	{
		Incidence * i = entry->incidence();
		kdDebug() << "CalendarDatabase: writing back incidence " 
				<< i->summary() << endl;
	}
	return NULL;
}
