/*
Copyright goes here.
*/

#include <qsocket.h>
#include <qregexp.h>
#include <qfile.h>


#include <kdebug.h>

#include "mlhttpserver.h"
#include "mlsession.h"
#include "mlsessionhandler.h"
#include "addressbookdatabase.h"
#include "calendardatabase.h"

#include "syncml.h"
#include "syncmldevinf.h"
#include "syncmldatabase.h"

#include <string>

#undef DEBUG_HEADER_DATA
#undef DEBUG_SESSION_DATA

// "dump" means to the ksyncml.log file, not standard output
#define DUMP_SESSION_DATA

MLHttpServer::MLHttpServer( QObject* parent):
    QServerSocket(8080,1,parent), mlSession(NULL), currentConnection(NULL), debugStream(NULL)
{
	#ifdef DEBUG_MEM
	kdDebug() << "DEBUG_MEM - MLHttpServer: created." << endl;
	#endif
	debugStream = NULL;
}

bool MLHttpServer::initialize()
{
	if ( !ok() )
	{
		kdWarning(5201) << "Failed to bind to port 8080"  << endl;
		return false;
	}
	else
	{
		kdDebug(5201) << "Server alive and bound to port 8080"  << endl;
		status = IDLE;
 
		#ifdef DUMP_SESSION_DATA
		QFile * debugFile = new QFile( "ksyncml.log" );
		if ( debugFile->open( IO_WriteOnly ) )
			debugStream = new QTextStream( debugFile );
		else
			kdWarning(5201) << "CANNOT OPEN DUMP FILE : ./ksyncml.log" << endl;
		#endif  
		
		return true;
	}
}

MLHttpServer::~MLHttpServer()
{
	delete mlSession;
	
	#ifdef DUMP_SESSION_DATA
	if (debugStream)
	{
		delete debugStream->device();
		delete debugStream;
	}
	#endif
	
	#ifdef DEBUG_MEM
	kdDebug() << "DEBUG_MEM - MLHttpServer: destroyed." << endl;
	#endif
	
	
}

void MLHttpServer::discardClient()
{
  QSocket* socket = (QSocket*)sender();
  delete socket;
}

void MLHttpServer::newConnection( int socket )
{
	//TODO: handle correctly http sessions (with connection/disconnection).
	delete currentConnection;
	
	currentConnection = new QSocket( this );
	connect( currentConnection, SIGNAL(readyRead()), this, SLOT(readClient()) );
	connect( currentConnection, SIGNAL(delayedCloseFinished()), this, SLOT(discardClient()) );
	currentConnection->setSocket( socket );

	
	status = RECEIVING_HEADER;//RESET.
	headerStr = "";
	
	
	kdDebug(5201) << "Initiating mlSession" << endl;

	delete mlSession;
	
	QString sourceAddr = "http://" + currentConnection->address().toString() 
			+ ":" + QString::number(currentConnection->port());
	SMLURI source(sourceAddr,"KitchenSync");

	//XXX FIXME I think the peer will send us his address, so no need
	//to set it here.
	QString destAddr = "http://" + currentConnection->peerAddress().toString() 
			+ ":" + QString::number(currentConnection->peerPort());
	SMLURI target(destAddr,"KitchenSync");

	SyncMLSession * session = new SyncMLSession(source, target, this, UNKNOWN);
	
	emit newSession(session->getHandler());
		
	kdDebug(5201) << "New Connection" << endl;
}


void MLHttpServer::readClient()
{
	#if defined(DEBUG_HEADER_DATA) || defined(DEBUG_SESSION_DATA)
	kdDebug(5201) << "HTTPServer: read event while status is " << status << endl;
	#endif

	if (!currentConnection)
	{
		kdWarning(5201) << "HTTPServer: received a data from a non existing socket!" << endl;
		return;
	}
		
	if ( status == RECEIVING_HEADER )
	{
		while ( (status == RECEIVING_HEADER) && (currentConnection->canReadLine()) )
		{
			QString theLine = currentConnection->readLine();
			#ifdef DEBUG_HEADER_DATA
			kdDebug(5201) << "HTTPServer: read: '"
				 << theLine.left(theLine.length()-2) 
				 << "' (" << theLine.length() << " bytes)" << endl;
			#endif
			
			if (theLine.length() == 2) //CR/LF
			{
				QHttpRequestHeader header(headerStr);
				if (header.isValid())
				{
					//XXX TODO Handle "Expect" header correctly.
					QTextStream os( currentConnection );
      					os.setEncoding( QTextStream::UnicodeUTF8 );
					os << "HTTP/1.1 100 Continue\r\n";
					os << "\r\n";

					QString tmp = header.value("Content-Length");
					if (tmp.isEmpty())
					{
						kdWarning(5201) << "HTTPServer: header received is missing Content-Length. Aborting"
							 << endl;
						discardClient();
						status = IDLE;
						return;
					}
					bufferSize = tmp.toULong();
					bufferReceived = 0;
					////recvBuffer.resize(bufferSize + 1);
					recvBuffer.resize(bufferSize);
					
					kdDebug(5201) << "HTTPServer: received a valid header. Waiting for "
					<< bufferSize - 1 << " bytes of data." << endl;	
					
					
					status = RECEIVING_DATA;
					
					//there can be some data already waiting
					readClient();
					
				}
				else
				{
					//TODO send an error code and discard the client
					kdWarning(5201) << " Syntax error. Closing." << endl;
					currentConnection->close();
					//delete currentConnection;
					//currentConnection = NULL;
				}
			}
			else
				headerStr = headerStr + theLine;
		}
		return;
	}
	
	if ( status == RECEIVING_DATA )
	{
		//TODO any ways of handling assert errors?
		if (bufferReceived > bufferSize)
			kdWarning(5201) << "bufferReceived > bufferSize even before reading!";
			
		bufferReceived += currentConnection->readBlock(recvBuffer.data() + bufferReceived, bufferSize - bufferReceived);

		#ifdef DEBUG_SESSION_DATA
		kdDebug(5201) << bufferReceived << " bytes out of " << bufferSize << " received so far" << endl;
		#endif
		
		if (bufferReceived > bufferSize)
			kdWarning(5201) << "bufferReceived > bufferSize!";
		else if (bufferReceived == bufferSize)
		{
			////recvBuffer[bufferSize] = (unsigned char)0;
			
			#ifdef DUMP_SESSION_DATA
			(*debugStream) << endl << endl << "------ Data TO the session object ------" << endl;
			(*debugStream) << QString(recvBuffer);
			(*debugStream) << endl;
			debugStream->device()->flush();
			#endif

			emit newData(recvBuffer);
			
			//mlSession->handleIncommingData(recvBuffer);
		}
		return;
	}
}

void MLHttpServer::disconnectClient()
{
	if (currentConnection->bytesToWrite() > 0)
	{
		kdDebug() << "MLHttpServer: writing " << currentConnection->bytesToWrite() << " bytes of data before closing." << endl;
		QObject::connect(currentConnection, SIGNAL(delayedCloseFinished()),
				this, SLOT(disconnected()));
		
		QObject::connect(currentConnection, SIGNAL(connectionClosed()),
				 this, SLOT(disconnected()));
				
		currentConnection->close();
		
		//XXX FIXME: OK, I don't know why delayedCloseFinished is never called.
		disconnected();
	}
	else
	{
		kdDebug() << "MLHttpServer: closing immediately." << endl;
		currentConnection->close();
		disconnected();
	}
}

void MLHttpServer::disconnected()
{
	emit clientDisconnected();
}

void MLHttpServer::sendData(QByteArray & response)
{
	unsigned long size = response.size();

	if (size == 0)
	{
		kdDebug(5201) << " BUG! Session wants us to send 0 bytes of data. Gone crazy?"<< endl;
	}
	else
	{
		kdDebug(5201) << " Sending the header and the session data (" << size << " bytes)"<< endl;
		QTextStream * oStream = new QTextStream((QIODevice *)currentConnection);
		(*oStream) << "HTTP/1.1 200 OK\r\n";	
		(*oStream) << "Content-Length: " << size << "\r\n";	
		(*oStream) << "Content-Type: application/vnd.syncml+xml\r\n";
		(*oStream) << "Accept-Charset: UTF-8\r\n";
		(*oStream) << "Cache-Control: no-store\r\n";
		//XXX TODO test wbxml and send "application/vnd.syncml+wbxml" too.
		(*oStream) << "Accept: application/vnd.syncml+xml\r\n";
		(*oStream) << "Server: KitchenSync\r\n";
		(*oStream) << "\r\n";
		
		(*oStream) << QString(response);
		(*oStream) << endl;
		
		#ifdef DUMP_SESSION_DATA
		(*debugStream) << endl << endl << "------ Data FROM the session object ------" << endl;
		(*debugStream) << QString(response);
		(*debugStream) << endl;
		debugStream->device()->flush();
		#endif
	}
	//kdDebug() << QString(response) << endl << endl;
	
	status = RECEIVING_HEADER;
	headerStr = "";
	
}

#include "mlhttpserver.moc"


