#ifndef KSYNC_MLKONNECTOR_H
#define KSYNC_MLKONNECTOR_H

#include <konnector.h>
#include <synceelist.h>
#include <addressbooksyncee.h>

#include <qiconset.h>
#include <qptrlist.h>

class SMLSessionHandler;


namespace KSyncML
{
class SyncMLDatabase;
class AddressBookDatabase;
class CalendarDatabase;
};


namespace KSync {

class MLKonnectorConfig;

// XXX FIXME The MLKonnector has a design bug. The konnector
// starts to read syncees as soon as connectDevice gets called.
// And it emits the signal even if readSyncees() was not called!


class MLKonnector : public KSync::Konnector
{ 
Q_OBJECT

public:
	MLKonnector( const KConfig *config );
	~MLKonnector();

	void writeConfig( KConfig * );

	/** return our capabilities() */
	//KSync::Kapabilities capabilities();

	/** the user configured this konnector
	* apply his preferecnes
	*/
	//void setCapabilities( const KSync::Kapabilities& );

	SynceeList syncees() { return mSyncees; }

	bool readSyncees();
	bool writeSyncees();

	bool connectDevice();
	bool disconnectDevice();

	/** the state and some informations */
	KSync::KonnectorInfo info()const;

	/** download a resource/url/foobar */
	void download( const QString& );
	
	virtual QStringList supportedFilterTypes() const;


private:
	MLKonnectorConfig *mConfigWidget;
	
	SynceeList mSyncees;

	//XXX FIXME: design problem.
	//it's not sure we need the databases here, after all. the session handler
	//should take care of everything and inform the konnector
	//when there is a data available and when the data has been written
	//to the other side and the conversation finished
	//XXX 2004-10-23: Start the implementation of the above.
	//KSyncML::AddressBookDatabase *mAddressBookDatabase;
	//KSyncML::CalendarDatabase * mCalendarDatabase;
	unsigned int databasesToRead;
	unsigned int databasesToWrite;
	
	class Private;
	Private *d;    
    
signals:
	//those signals are used to communicate with the databases
	void dbReadSyncees();//not used at the moment
	void dbWriteSyncees(SMLSessionHandler *);
	
private slots:
	void slotNewSession(SMLSessionHandler *);
	void slotSynceesRead(KSyncML::SyncMLDatabase *);
	void emitSynceesRead();
	void emitSynceesWritten();
	void slotSynceesWritten(KSyncML::SyncMLDatabase *);
	void slotConversationFinished();
	
	void doWriteSyncees();
	void doReadSyncees();
};

}


#endif
