#ifndef SYNCMLSESSION_H
#define SYNCMLSESSION_H

#include <qobject.h>
#include <qcstring.h>
#include <qptrlist.h>
#include <qmap.h>

#include <syncentry.h>

#include "syncml.h"

class QTextStream;
class SMLSessionHandler;
class SMLURI;
class MLHttpServer;


class SyncMLItem
{
public:
	QString ID;
	QString target;
	QString source;
	int msgRef;
	int cmdRef;
	QString item;
	QString syncEntryID;
	enum {ADDED, REPLACED, REMOVED} action;
};

class SyncMLSession: public QObject
{
Q_OBJECT

public:
	typedef QValueList<SyncMLItem> TSynceesList;
	typedef QMap<QString, QString> TKeysList;
	
	/**
	* Creates a new session for the specified target using the specified communication layer
	* with a specified protocol version.
	* PS: SMLURI source is unnecessary. MLHttpServer will become a generic interface.
	*/
	SyncMLSession(SMLURI & source, SMLURI & target, MLHttpServer * communication, SMLVersion protocolVersion = UNKNOWN);
	virtual ~SyncMLSession();

	virtual void raiseError(SMLErrorType error, string description="");
	
	virtual SMLErrorType getLastErrorCode() {  return lastErrorCode;  };
	virtual bool hasError() { return (lastErrorCode != SMLError::NoError) && (lastErrorCode != SMLError::Successful); };	
	
	SMLSessionHandler * getHandler() { return handler; };

signals:
	void sessionFinishedStep();
	void sessionFinished();
	
	/** The list of items containing data for syncees for the given database. 
	  The receiver is supposed to know how to parse the actual data in the items.
	*/
	void synceesRead(SyncMLSession::TSynceesList &, SMLURI &);
	void keysRead(SyncMLSession::TKeysList &, SMLURI &);

	
	
protected slots:
	/* Parse the data */
	void handleIncommingData(QByteArray & data);
	
	
	/** should get called at the beginning of the tag */
	virtual void willReceiveEvent(SMLTokenType event);
	
	/** an Event gets fired once the approriate tag is complete */
	virtual void eventReceived(SMLTokenType event, SMLTreeNode *CurrentNode);
	void parserFinished();
	void emitSessionFinishedStep();
	void emitSynceesRead();
	void emitKeysRead();

private:
	SMLErrorType lastErrorCode;
	string lastErrorString;

	SyncMLParser *parser;
	
	
	SMLSessionHandler *handler;
	bool isServer;
	SMLUInt currentMsgRef;
	SMLAlertType syncType;

	bool inLater;
	queue<SMLTreeNode*> doLater;
	
	
	/** The problem is that we cannot know what data we are syncing
	  until we receive the complete xml "Sync" tag. Therefore, we
	  need to temporarily store the received data (only strings + statuses) here
	  and fire the "synceesRead" signal at the end when we will know the URI.
	  */
	TSynceesList synceesList;
	SMLURI *synceesDatabase;
	
	/** The same for keys data
	*/
	TKeysList keysList;
	SMLURI *keysDatabase;
	
	/** Is true, if we are receiving the 
	actual synchronization data (we are in the middle of <Sync> </Sync>)
	*/
	bool syncingInProgress;
	SMLTreeNode* syncNode;
	
};

#endif
