/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#ifndef __DEVINF_H
#define __DEVINF_H

#include "syncml.h"
#include "syncmldatabase.h"

class SMLTreeNode;

class SMLDevInf
{
 protected:
  SMLTreeNode *headNode;
  void realInit();
  void initFromTreeNode(SMLTreeNode *otherRoot);
  SMLVersion syncmlVersion;
 public:
  struct Param {
    std::string Name;
    std::list<std::string> ValEnum;
    std::string DataType;
    SMLUInt Size;
    std::string DisplayName;
    Param() : Name(), ValEnum(), DataType(), DisplayName() { 
      Size=0;
    }
  };
  struct Prop : Param {
    std::list<Param> ParamList;
    Prop() : Param(),ParamList() {};
  };
  struct CTCap {
    std::string CTType;
    std::list<Prop> PropList;
    CTCap() : CTType(), PropList() {}
  };
  
  // Constructors
  SMLDevInf();
  SMLDevInf(SMLTreeNode *t);
  SMLDevInf(SMLDevInf *s);

  // Somehow internal functions
  virtual void eventReceived(SMLTokenType event, SMLTreeNode *CurrentNode);

  // Action functions
  void save(QTextStream *o);
  void enforceConstraints();
  void enforceConstraints(SMLTreeNode *node);

  SMLTreeNode *getDevInfTree();
  void addDevInfToPacket(SMLTreeNode *n);

  // Atribute functions
  std::string getDevId();
  void setDevId();
  void setDevId(std::string DevId);

  enum DefTypEnum {pager,handheld,pda,phone,smartphone,server,workstation};
  std::string getDevTyp();
  void setDevTyp(DefTypEnum e);
  void setDevTyp(std::string s);

  std::string getFwV();
  void setFwV(std::string s);

  std::string getHwV();
  void setHwV(std::string s);

  std::string getMan();
  void setMan(std::string s);

  std::string getMod();
  void setMod(std::string s);

  std::string getOEM();
  void setOEM(std::string s);

  std::string getSwV();
  void setSwV(std::string s);

  std::string getVerDTD();
  void setVerDTD(SMLVersion version);

  bool hasExt(std::string extName);
  void setExtValues(std::string extName, vector<std::string> extValues);
  std::vector<std::string> getExtValues(std::string extName);
  std::vector<std::string> getExts();
  void deleteExt(std::string extName);

  //XXX FIXME this is in plain refractoring.
  bool hasDataStoreInfo(std::string SourceRef);
  void addDataStoreInfo(KSyncML::SyncMLDatabase::DataStoreType & values);
  KSyncML::SyncMLDatabase::DataStoreType getDataStoreInfo(std::string sourceRef);
  std::vector<std::string> getDataStores();
  void deleteDataStoreInfo(std::string dataStore);

  bool hasCTCap(std::string CTType);
  void setCTCap(CTCap values);
  CTCap getCTCap(std::string CTType);
  std::vector<std::string> getCTCaps();
  void deleteCTCap(std::string CTType);

  // Destructor
  virtual ~SMLDevInf();
};

#endif


