/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#ifndef __SMLRESPONSEPACKET_H
#define __SMLRESPONSEPACKET_H

#include "syncml.h"
#include "syncmluri.h"
#include "xmlparser_tree.h"
#include "xmlnamespace.h"

class SMLResponsePacket {
 private:
  SMLTreeNode *headNode;
  SMLTreeNode *header;
  SMLTreeNode *body;
  SMLNamespaceContainer *nsCont;
  SMLUInt messageID;
  SMLUInt CmdID;

  SMLURI sourceDev;
  SMLURI targetDev;

  SMLVersion syncmlVersion;
  
  void reinitialize();
  bool sendAtAll;

 public:
  SMLResponsePacket();
  ~SMLResponsePacket();
  
  //you must call this exactly once.
  void setSyncMLVersion(SMLVersion protocolVersion);
  
  SMLNamespaceContainer* getNamespaceContainer() { return nsCont; }
  bool send(SMLFlattener *flattener);

  void setSessionID(string sessionID);
  string getSessionID();

  SMLURI* sourceDevice() { return &sourceDev; }
  SMLURI* targetDevice() { return &targetDev; }

  //  SMLTreeNode* addMsgRefAndCmdRef(SMLUInt MsgRef, SMLTreeNode *nodeRef);
  SMLTreeNode* addSubCommand(SMLTreeNode* node, SMLTokenType t, SMLCmdRef* cmdid, bool vitallyForSend=true);
  SMLTreeNode* addCommand(SMLTokenType t, SMLCmdRef* cmdid, bool vitallyForSend=true);
  SMLTreeNode* addStatus(SMLUInt MsgRef, SMLTreeNode *nodeRef, SMLErrorType statusCode); 
  SMLTreeNode* addResults(SMLUInt MsgRef, SMLTreeNode *nodeRef);
  
  SMLTreeNode* addStatus(SMLUInt MsgRef, SMLUInt CmdRef, QString cmd, SMLErrorType statusCode);

  void forceSend()
  {
	  sendAtAll = true;
  }
  
};

#endif
