/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#include "syncml.h"
#include "xmlbeautifier.h"
#include "xmlnamespace.h"
#include "xmlbinary.h"
#include <qtextstream.h>

#undef QT_NO_STL

#include <qstring.h>


using namespace std;

string SyncMLFlattener::addNamespace(SMLTokenSpaceType mySpace,
                                     SMLTokenSpaceType parent) 
{
  if (mySpace==parent) return "";
  return (" xmlns='"+namespaceContainer->getSyncMLStringForNamespace(mySpace)+"'");
}

bool SyncMLFlattener::header() {  
  // FIXME
  // Maybe output <?xml> and <!DOCTYPE syncml> ???
  // But the spec doens't require, and sometimes doesn't even want that.
  // But we keep the funtion in here for now. Someday we might actually need it.
   *outputStream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r";
  // *outputStream << "<!DOCTYPE SyncML PUBLIC \"-//SYNCML//DTD SyncML 1.0//EN\"
  // \"http://www.syncml.org/docs/syncml_represent_v10_20001207.dtd\">\r";
  return true;
}

string SyncMLFlattener::buildStartTag(SMLTokenType token, SMLTokenSpaceType parent,
				      string *s) {
  string ret;
  if (token.token == SMLToken::AlienToken) 
    ret = *s + addNamespace(token.space,parent);
  else
    ret = WBXTranslator::getSyncMLStringForToken(token) + addNamespace(token.space,parent);
  return ret;
 }

bool SyncMLFlattener::startTag(SMLTokenType token, SMLTokenSpaceType parent,
                               string *s) {
  *outputStream << "<" << QString(buildStartTag(token,parent,s).c_str()) << ">";
  return true;
}

bool SyncMLFlattener::endTag(SMLTokenType token, string *s) {
  if (token.token == SMLToken::AlienToken) 
    *outputStream << "</" << QString(string(*s,0,s->find(" ")).c_str()) << ">";
  else
    *outputStream << "</" << QString(WBXTranslator::getSyncMLStringForToken(token).c_str()) << ">";
  return true;
}

bool SyncMLFlattener::text(string *s) {
  *outputStream << QString(s->c_str());
  return true;
}

NiceSyncMLFlattener::NiceSyncMLFlattener(QTextStream *o, SMLNamespaceContainer
                                         *ns, SMLUInt indent) :
  SyncMLFlattener(o,ns) {
  
  indentBy=indent;
  last=close;
  indentCount=0;
}

bool NiceSyncMLFlattener::startTag(SMLTokenType token, SMLTokenSpaceType
                                   parent, string *s) {
  if (last==open) *outputStream << endl;
  ////
  if (last!=txt) *outputStream << QString(string(indentCount,' ').c_str());
  bool ret = SyncMLFlattener::startTag(token,parent,s);
  indentCount+=indentBy;
  last = open;
  return ret;
}

bool NiceSyncMLFlattener::endTag(SMLTokenType token, string *s) {
  indentCount-=indentBy;
  if (last==close) *outputStream << QString(string(indentCount,' ').c_str());
  bool ret = SyncMLFlattener::endTag(token,s);
  *outputStream << endl;
  last = close;
  return ret;
}

bool NiceSyncMLFlattener::emptyTag(SMLTokenType token, SMLTokenSpaceType parent, string *s) {
  if (last==open) *outputStream << endl;
  if (last!=txt) *outputStream << QString(string(indentCount,' ').c_str());
  *outputStream << "<" << QString(buildStartTag(token,parent,s).c_str()) << "/>";
  *outputStream << endl;
  last = close;
  return true;
}

bool NiceSyncMLFlattener::text(string *s) {
  bool ret = SyncMLFlattener::text(s);
  last = txt;
  return ret;
}


