/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "syncml.h"
#include "xmlbinary.h"

#include <map>

WBXTranslator * WBXTranslator::singleton = NULL;

WBXTranslator::WBXTranslator()
{}

void  WBXTranslator::addMapping(string s, SMLTokenType t)
{
	StringForToken[t] = s;
	if (t.space==SMLNamespace::SyncML) SyncMLToken[s]=t; else
			if (t.space==SMLNamespace::MetInf) MetInfToken[s]=t; else
			if (t.space==SMLNamespace::DevInf) DevInfToken[s]=t;
}

void WBXTranslator::WBXMLInitialize()
{
	singleton = new WBXTranslator();
	// SyncML Namespace
	singleton->addMapping("Add",SMLToken::Add);
	singleton->addMapping("Alert",SMLToken::Alert);
	singleton->addMapping("Archive",SMLToken::Archive);
	singleton->addMapping("Atomic",SMLToken::Atomic);
	singleton->addMapping("Chal",SMLToken::Chal);
	singleton->addMapping("Cmd",SMLToken::Cmd);
	singleton->addMapping("CmdID",SMLToken::CmdID);
	singleton->addMapping("CmdRef",SMLToken::CmdRef);
	singleton->addMapping("Copy",SMLToken::Copy);
	singleton->addMapping("Cred",SMLToken::Cred);
	singleton->addMapping("Data",SMLToken::Data);
	singleton->addMapping("Delete",SMLToken::Delete);
	singleton->addMapping("Exec",SMLToken::Exec);
	singleton->addMapping("Final",SMLToken::Final);
	singleton->addMapping("Get",SMLToken::Get);
	singleton->addMapping("Item",SMLToken::Item);
	singleton->addMapping("Lang",SMLToken::Lang);
	singleton->addMapping("LocName",SMLToken::LocName);
	singleton->addMapping("LocURI",SMLToken::LocURI);
	singleton->addMapping("Map",SMLToken::Map);
	singleton->addMapping("MapItem",SMLToken::MapItem);
	singleton->addMapping("Meta",SMLToken::Meta);
	singleton->addMapping("MsgID",SMLToken::MsgID);
	singleton->addMapping("MsgRef",SMLToken::MsgRef);
	singleton->addMapping("NoResp",SMLToken::NoResp);
	singleton->addMapping("NoResults",SMLToken::NoResults);
	singleton->addMapping("Put",SMLToken::Put);
	singleton->addMapping("Replace",SMLToken::Replace);
	singleton->addMapping("RespURI",SMLToken::RespURI);
	singleton->addMapping("Results",SMLToken::Results);
	singleton->addMapping("Search",SMLToken::Search);
	singleton->addMapping("Sequence",SMLToken::Sequence);
	singleton->addMapping("SessionID",SMLToken::SessionID);
	singleton->addMapping("SftDel",SMLToken::SftDel);
	singleton->addMapping("Source",SMLToken::Source);
	singleton->addMapping("SourceRef",SMLToken::SourceRef);
	singleton->addMapping("Status",SMLToken::Status);
	singleton->addMapping("Sync",SMLToken::Sync);
	singleton->addMapping("SyncBody",SMLToken::SyncBody);
	singleton->addMapping("SyncHdr",SMLToken::SyncHdr);
	singleton->addMapping("SyncML",SMLToken::SyncML);
	singleton->addMapping("Target",SMLToken::Target);
	singleton->addMapping("TargetRef",SMLToken::TargetRef);
	singleton->addMapping("Reserved",SMLToken::Reserved);
	singleton->addMapping("VerDTD",SMLToken::VerDTD);
	singleton->addMapping("VerProto",SMLToken::VerProto);

	// MetaInf Namespace
	singleton->addMapping("Anchor",SMLToken::MetInf::Anchor);
	singleton->addMapping("EMI",SMLToken::MetInf::EMI);
	singleton->addMapping("Format",SMLToken::MetInf::Format);
	singleton->addMapping("FreeID",SMLToken::MetInf::FreeID);
	singleton->addMapping("FreeMem",SMLToken::MetInf::FreeMem);
	singleton->addMapping("Last",SMLToken::MetInf::Last);
	singleton->addMapping("Mark",SMLToken::MetInf::Mark);
	singleton->addMapping("MaxMsgSize",SMLToken::MetInf::MaxMsgSize);
	singleton->addMapping("Mem",SMLToken::MetInf::Mem);
	singleton->addMapping("MetInf",SMLToken::MetInf::MetInf);
	singleton->addMapping("Next",SMLToken::MetInf::Next);
	singleton->addMapping("NextOnce",SMLToken::MetInf::NextOnce);
	singleton->addMapping("SharedMem",SMLToken::MetInf::SharedMem);
	singleton->addMapping("Size",SMLToken::MetInf::Size);
	singleton->addMapping("Type",SMLToken::MetInf::Type);
	singleton->addMapping("Version",SMLToken::MetInf::Version);

	// DevInf Namespace
	singleton->addMapping("CTCap",SMLToken::DevInf::CTCap);
	singleton->addMapping("CTType",SMLToken::DevInf::CTType);
	singleton->addMapping("DataStore",SMLToken::DevInf::DataStore);
	singleton->addMapping("DataType",SMLToken::DevInf::DataType);
	singleton->addMapping("DevId",SMLToken::DevInf::DevId);
	singleton->addMapping("DevInf",SMLToken::DevInf::DevInf);
	singleton->addMapping("DevTyp",SMLToken::DevInf::DevTyp);
	singleton->addMapping("DisplayName",SMLToken::DevInf::DisplayName);
	singleton->addMapping("DSMem",SMLToken::DevInf::DSMem);
	singleton->addMapping("Ext",SMLToken::DevInf::Ext);
	singleton->addMapping("FwV",SMLToken::DevInf::FwV);
	singleton->addMapping("HwV",SMLToken::DevInf::HwV);
	singleton->addMapping("Man",SMLToken::DevInf::Man);
	singleton->addMapping("MaxGUIDSize",SMLToken::DevInf::MaxGUIDSize);
	singleton->addMapping("MaxID",SMLToken::DevInf::MaxID);
	singleton->addMapping("MaxMem",SMLToken::DevInf::MaxMem);
	singleton->addMapping("Mod",SMLToken::DevInf::Mod);
	singleton->addMapping("OEM",SMLToken::DevInf::OEM);
	singleton->addMapping("ParamName",SMLToken::DevInf::ParamName);
	singleton->addMapping("PropName",SMLToken::DevInf::PropName);
	singleton->addMapping("Rx",SMLToken::DevInf::Rx);
	singleton->addMapping("Rx-Pref",SMLToken::DevInf::RxPref);
	singleton->addMapping("SharedMem",SMLToken::DevInf::SharedMem);
	singleton->addMapping("Size",SMLToken::DevInf::Size);
	singleton->addMapping("SourceRef",SMLToken::DevInf::SourceRef);
	singleton->addMapping("SwV",SMLToken::DevInf::SwV);
	singleton->addMapping("SyncCap",SMLToken::DevInf::SyncCap);
	singleton->addMapping("SyncType",SMLToken::DevInf::SyncType);
	singleton->addMapping("Tx",SMLToken::DevInf::Tx);
	singleton->addMapping("Tx-Pref",SMLToken::DevInf::TxPref);
	singleton->addMapping("ValEnum",SMLToken::DevInf::ValEnum);
	singleton->addMapping("VerCT",SMLToken::DevInf::VerCT);
	singleton->addMapping("VerDTD",SMLToken::DevInf::VerDTD);
	singleton->addMapping("XNam",SMLToken::DevInf::XNam);
	singleton->addMapping("XVal",SMLToken::DevInf::XVal);

}

SMLTokenType WBXTranslator::getWBXMLTokenForString(string s, SMLTokenSpaceType space)
{
	if (!singleton)
		WBXMLInitialize();
		
	// If not in a special Namespace assume we're in the syncml Namespace
	if (space==SMLNamespace::None) return singleton->SyncMLToken[s];
	if (space==SMLNamespace::SyncML) return singleton->SyncMLToken[s];
	if (space==SMLNamespace::MetInf) return singleton->MetInfToken[s];
	if (space==SMLNamespace::DevInf) return singleton->DevInfToken[s];
	if ((space>=SMLNamespace::OtherStart) && (space<=SMLNamespace::OtherEnd))
	{
		SMLTokenType t;
		t.space = space;
		t.token = SMLToken::AlienToken;
		return t;
	}
	return SMLToken::Illegal;
}

string WBXTranslator::getSyncMLStringForToken(SMLTokenType token)
{
	if (!singleton)
		WBXMLInitialize();
		
	return singleton->StringForToken[token];
}


