/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>
#include "syncml.h"

#include "xmlparser.h"
#include "xmlparser_tree.h"
#include "xmlbinary.h"
#include "xmlnamespace.h"

#include <qstring.h>
#include <qtextstream.h>
#include <kdebug.h>

#undef PARSER_DEBUG

SyncMLParser::SyncMLParser(QByteArray *dataToParse):QObject(NULL, ""), QXmlDefaultHandler(), xmlData(dataToParse)
{
	kdDebug() << "SyncMLParser: created." << endl;
	//at this moment we cannot know what is the protocol version
	namespaceContainer = new SMLNamespaceContainer(UNKNOWN); 
	HeadNode = new SMLTreeNode(NULL,SMLToken::Root);
	CurrentNode = HeadNode;	
}

SyncMLParser::~SyncMLParser() 
{
  if (HeadNode!=NULL) delete HeadNode;
  delete namespaceContainer;
}

bool SyncMLParser::startElement( const QString& namespaceURI, const QString& localName,
                                    const QString& qName, const QXmlAttributes& attributes)
{
	(void)localName;
	(void)attributes;
	#ifdef PARSER_DEBUG
	kdDebug() << "SyncMLParser: new element: " << qName << " of namespace " << namespaceURI << endl;
	#endif
	
	string name(qName.ascii());
	
	//XXX FIXME: ugh, what a hack :+(
	QString spaceURI = namespaceURI;
	if (spaceURI == "")
		spaceURI = "SYNCML:SYNCML1.1";
	
	SMLTokenSpaceType space = getNamespaceContainer()->addNewNamespace(spaceURI);
	//XXX TODO MANAGE Other "alien" attributes
	// Pare Attributes. SyncML doesn't use them, but some embedded XML might. 
	// We're just interested in the namespaces, everything else we don't care.
  
  
	SMLTokenType token = WBXTranslator::getWBXMLTokenForString(name,space);
	//SMLDBG("Start: " << token);

	string tokenStr = "";
/*
  if (token.token==SMLToken::AlienToken) {
    tokenStr = string((const char*)name)+attrstring;
    //SMLDBG("Alien token: " << tokenStr);
  }
*/	
	SMLTreeNode *newNode = new SMLTreeNode(CurrentNode,token,tokenStr);
	CurrentNode = newNode;
	
	emit futureEvent(token);
	
	
	return TRUE;
}

bool SyncMLParser::characters ( const QString & ch )
{
	(void)new SMLTreeNode(CurrentNode,SMLToken::String, ch);	
	return TRUE;
}

bool SyncMLParser::endElement( const QString&, const QString& localName, const QString& qName)
{
	(void)localName;
	#ifdef PARSER_DEBUG
	kdDebug() << "SyncMLParser: end element: " << qName << endl;
	#endif

	const char * name = qName.latin1();
  
	SMLTokenSpaceType space = CurrentNode->getType().space;
	SMLTokenType token = WBXTranslator::getWBXMLTokenForString(string((const char*)name),space);

	#ifdef PARSER_DEBUG
	kdDebug() << "/" << token <<  " should be " << CurrentNode->getType() << endl;
	#endif

	//MLObject->eventReceived(token,CurrentNode);
	emit newEvent(token, CurrentNode);

	SMLTreeNode* upperNode = CurrentNode->getParent();
	CurrentNode = upperNode;
	
	//SMLDBG("Returning to " << CurrentNode->getType());
	return TRUE;
}

bool SyncMLParser::startDocument()
{
	return TRUE;
}

bool SyncMLParser::endDocument()
{
	emit endParse();
	return TRUE;
}


bool SyncMLParser::warning ( const QXmlParseException & exception )
{
	kdDebug() << "SyncMLParser: warning '" 
		<< exception.message() << "' at line/col " 
		<< exception.lineNumber() << "/" << exception.columnNumber() << endl;
	return TRUE;
}

bool SyncMLParser::error ( const QXmlParseException & exception )
{
	kdDebug() << "SyncMLParser: error '" 
		<< exception.message() << "' at line/col " 
		<< exception.lineNumber() << "/" << exception.columnNumber() << endl;
	return TRUE;
}

bool SyncMLParser::fatalError ( const QXmlParseException & exception )
{
	kdDebug() << "SyncMLParser: fatal error '" 
		<< exception.message() << "' at line/col " 
		<< exception.lineNumber() << "/" << exception.columnNumber() << endl;
	return TRUE;
}

QString SyncMLParser::errorString ()
{
	return lastError;
}


void SyncMLParser::startParse()
{
	//XXX TODO handle xml errors correctly
	if (xmlData)
	{
		kdDebug() << "SyncMLParser: trying to parse something (" << xmlData->count() << " bytes)" << endl;
		
		QFile ala("syncmlparser.dump");
		//ala.open(IO_ReadOnly);
		//ala.open(IO_WriteOnly);
		//ala.writeBlock(*xmlData);
		//ala.flush();
		//ala.close();
		//xmlSource = new QXmlInputSource(&ala);
		
		xmlSource = new QXmlInputSource();
		xmlSource->setData(*xmlData);
		
		
		xmlReader.setContentHandler( this );
		xmlReader.setErrorHandler( this );
		//XXX TODO send a signal about failed parsing.
		if (!xmlReader.parse(xmlSource, FALSE))
			kdWarning() << "SyncMLParser: Parse failed with error message: " << errorString() << endl;
			
			
		kdDebug() << "SyncMLParser: Parsing finished" << endl;
		return;
	}
	else
		kdWarning() <<"SyncMLParser: no data to parse" << endl;
}

#include "xmlparser.moc"
