/***************************************************************************
                                 libSyncML
           Implementation of the SyncML protocol (www.syncml.org)
         Copyright (C) 2001, 2002 by Max Berger <max.berger@xslt.de>
 **************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Lesser General Public             *
 *  License as published by the Free Software Foundation; either           *
 *  version 2.1 of the License, or (at your option) any later              *
 *  version.                                                               *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Lesser General Public License for more details.                        *
 *                                                                         *
 *  You should have received a copy of the GNU Lesser General Public       *
 *  License along with this library; if not, write to the Free             *
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA      *
 *  02111-1307 USA                                                         *
 *                                                                         *
 ***************************************************************************/

#ifndef __PARSER_H
#define __PARSER_H


#include <qobject.h>
#include <qxml.h>
#include <qcstring.h>

class QString;

#include "xmlparser_tree.h"


class SyncMLParser: public QObject, public QXmlDefaultHandler
{
Q_OBJECT
private:
	SMLTreeNode* HeadNode;
	SMLTreeNode* CurrentNode;
	SMLNamespaceContainer *namespaceContainer;
	
	QString lastError;	
	QByteArray *xmlData;
	QXmlInputSource * xmlSource;
	QXmlSimpleReader xmlReader;
public:
	/** The pointer is not deleted by the parser at the destruction.*/
	SyncMLParser(QByteArray *dataToParse);
	virtual ~SyncMLParser();
	
	void startParse();
	
	//content handler
	virtual bool startDocument();
	virtual bool endDocument();

	virtual bool startElement( const QString&, const QString&, const QString& ,
                       const QXmlAttributes& );
	virtual bool characters ( const QString & ch );
	virtual bool endElement( const QString&, const QString&, const QString& );
  
	//error handler
	virtual bool warning ( const QXmlParseException & exception );
	virtual bool error ( const QXmlParseException & exception );
	virtual bool fatalError ( const QXmlParseException & exception );
	virtual QString errorString ();
	
  
	SMLTreeNode* getHeadNode() { return HeadNode; };
	SMLNamespaceContainer* getNamespaceContainer() { return namespaceContainer; }
	
signals:
	/**futureEvent is fired at the beginning of corresponding
	  tag, when no details are known yet */
	void futureEvent(SMLTokenType event);
	
	/** newEvent is fired when the event tag is completely received
	*/
	void newEvent(SMLTokenType event, SMLTreeNode *CurrentNode);
	
	/** The parser can be deleted in the slot of this signal.
	*/
	void endParse();
};


#endif
